#include <reg921.h>
#include <uart.h>
#include <tea5764.h>
#include <timeconvert.h>
#include <fmscan.h>
#include <timers.h>
#include <dcf.h>
#include <util.h>
#include <delay.h>



//prototypes
void initHW(void);			   //initialize HW
void no_touch921(void);	 		//set config to run bootloader on next reset
void HandleUserInput(void);	   //a char on serial port has been recived process that data


//data
bit ScanFm;			   //start scannung for a new radio station
bit rdsOn;			   //IF FM scan has found a station rdsOn goes 1 to activate data processing
bit SentRawRDS;			//If set the serial output will be the raw RDS data
bit SentDCF;			//default on if set a DCF77 like pulse stream is genneratet
bit SendDate;			//if set the date and time will be serialli send as HEX numbers

void main(void)
{
signed char ct_timeout=0;  //timeout to detect if a station sends RDS but no CT (Group 4A)
bit ConvertT=0;			   //new RDS CT data is avalible convert to local date and time
bit IncMin=0;			   //increment the minte by 1
bit StationSaved=0;			//set to 1 if a good station was saved to EPROM
bit ValidCTTime=0; 			//true DCF data will be genneratet
signed char RdsCnt=0;		 //used to detect a stable station for save
unsigned char RDS_TimeOut=0; //used to determin if radio station will not send RDS data
bit t_offsetSaved=0;		 //timer calibration is saved
unsigned char stableToffset=0;  //used to dertermin if timer calibration is stable and can be saved

//init data
ScanFm=1;					//Radio scan for next Statio
rdsOn=1;					// enable RDS decode
SentRawRDS=0;
SentDCF=1;					//default is to output a DCF like pulse stream
SendDate=0;



//initialize Hardware
initHW();

//enable interrupts
EA = 1;

TEA_Init();	//init TEA5764

while(1)
	{
	if(M_Tic)  //one Minue is over
		{
		//uart_transmit('M');
		M_Tic=0;
		if(RdsCnt >5) //check if we have a stable RDS station
			{
			 if(!StationSaved)
			 	{ // we got a stable radio station if not done  save frequency to EPROM
				TEASaveFrequenzy();
				StationSaved=1;
				}
			RdsCnt=2;
			}
		RdsCnt--;
		//at beginnig of minute gennerate DCF data
		IncMin=1;
		}

	if(S_Tic)  //one second is gone
		{
		S_Tic=0;
	   //RDS Timeout
		if(rdsOn)
			{
			if(RDS_Level<10)
				RDS_TimeOut++;
			else
			   RDS_TimeOut=0;
			RDS_Level=0;
			ct_timeout++;
			}
		else
			ct_timeout=0;

		if(ct_timeout>=80)	//no CT info in RDS data for a while find a new Station
			{
			rdsOn=0;
			ScanFm=1;
			uart_transmit('C');
			}
		if((RDS_TimeOut>30) && (RdsCnt==0))	 //Station does not send RDS data find next station
			{
			rdsOn=0;
			ScanFm=1;
			uart_transmit('R');
			}
		//DCF Signal genneration
		if (SentDCF)
			{
			if(ValidCTTime && second!=59) //send a pulse every second exept Second 59
				{
				bit dcfbit;
			 	dcfbit=GetDCFBit();	 	 //get the Bit to send
			 	DcfPulseOn=1;
			 	if(dcfbit)
					{
			 		DcfPulse=20;		 //a one is coded as 200ms pulse
					}
				else
					{
					DcfPulse=10;		 //a zero is coded as 100ms pulse
					}
				}
			else
				ValidCTTime=0;			//One minute gone disable sending till we got a new CT data package
			}
		//data processing
		if(ConvertT)				  // New CT data is avalible convert to time and date
			{
			ConvertT=0;
			DecodeRDSDateAndTime();	   
			ValidCTTime=1;
			IncMin=1;				   //the date send is the current minute for DCF we need the next minute for 
			if(SendDate)
				{
				SendAsHex(0xff);
				SendAsHex(0xff);
				SendAsHex(ByteToBCD(localYear));
				SendAsHex(ByteToBCD(localMonth));
				SendAsHex(ByteToBCD(localDay));
				SendAsHex(localWDay);
				SendAsHex(ByteToBCD(localHours));
				SendAsHex(ByteToBCD(localMinutes));
				SendAsHex(0xff);
				}
			}
		//inc Minutes by 1
		if(IncMin)
			{
			IncMin=0;
			if(!IncMinute())
				ValidCTTime=0; 		//inc will fail if it is 23:59
			if(ValidCTTime)
				{
				GenDCFData();		//gennerate the DCF77 data to send in Second 20 to 58
				}
			}
		if(!t_offsetSaved && stableToffset>10) //save the timer calibration if offset is stable
			{
			TimerSaveReload();
			t_offsetSaved=1;
			}
		} //end of S_Tic

	
	if(ScanFm)	//find a radio station
		{
		ScanFm=0;
		ct_timeout=0;
		StationSaved=0;
		RdsCnt=0;
		RDS_TimeOut=0;
		TEAScan(); 	//scan for a station
		rdsOn=1;
		}

	if(rdsOn)	   //RDS processing is on
		{
		 tea5764_read_int_registers();  //need to POLL interrupt status not enouht code space
		 if(tea5764_fm_buf[REG_INTFLAG]!=0)
		 	{
			 ProcessTEAInterrupt();
			}
		}

	if(CTava) //convert DateTime from RDS data  (MJD) to local date and time
		{
		signed int delta;
		CTava=0;
			ct_timeout=-120;
		delta=ResetS_Count();  //reset internal Timer for synchronisation
		if(delta<0)				//adjust internal timer
			{
			delta=0-delta;
			if(delta>=0 && delta <200)
				{
				Reload-=(delta>>1); //speed up the timer
				}
			}
		else
			{
			if(delta>=10 && delta <200)	  //slow down the timer
				{
				Reload+=(delta>>1);
				}
			}
		ConvertT=1;
		RdsCnt+=2;
		if(delta<=10 && delta>=-10)  //timer is precise enouth (hit one minue +/- 200ms)
			{
			ValidCTTime=1;			//allow sending DCF data pulses
			stableToffset++;
			}
		else
			{
			stableToffset=0;
			}

		}

//user interface
	if(uart_peek())
		{
		HandleUserInput();
		}
	}
}


//a char on serial port has been recived process that data
void HandleUserInput(void)
{
unsigned char by;  //char recived by serial
unsigned int tmp_frq;

	by=uart_get(); //get the char form UART
	uart_transmit(by);	   //echo back
	if(by=='U')			   //Enable Bootloader for Brogramming
		{
		no_touch921();
		uart_transmit('U');
		}
	if(by=='G')			   //return current frequency in 100khz increments
		{
		//get found frequency
		tmp_frq=tea5764_GetFreq();
		SendAsHex(tmp_frq>>8);
		SendAsHex(tmp_frq&0xFF);
		}

/*	if(by=='F')
	{
	tea5764_read_fm_registers();
	DumpFM();
	}*/

	if(by=='R')		 //switch to send the recived raw RDS data  
		{
		SentRawRDS=1;					
		SentDCF=0;
		SendDate=0;
		};
	if(by=='D')		 //switch to send the recived datetime as DCF pulse stream
		{
		SentRawRDS=0;
		SentDCF=1;
		SendDate=0;
		};
	if(by=='H')		   //switch to send the recived datetime as HEX stream 
		{
		SentRawRDS=0;
		SentDCF=0;
		SendDate=1;
		};
	if(by=='X')		//reste the device
		{
		AUXR1|=0x8; //reset
		}
	if(by=='S')		 //find new radio Statio
		{
		ScanFm=1;
		rdsOn=0;
		}
}