#include <i2c.h>
#include <tea5764.h>
#include <delay.h>
#include <flash.h>
#include <uart.h>


//the values to initialize the tea5764 FM registers 
code unsigned char FM_InitData[TEA5764_FM_WCOUNT2] ={INTMSK,FRQSETMSB,FRQSETLSB,TNCTRL1,TNCTRL2,TESTBITS,TESTMODE};
//the values to initialize the tea5764 RDS registers
code unsigned char RDS_InitData[TEA5764_RDS_WCOUNT] ={RDSCTRL1,RDSCTRL2,PAUSEDET,RDSBBL};
//frequenzy to start searching
code unsigned int Fr_Save=0x380; //874;

//FM register buffer
idata unsigned char tea5764_fm_buf[ TEA5764_FM_RCOUNT ];
//RDS register Buffer
idata unsigned char tea5764_rds_buf[ TEA5764_RDS_RCOUNT ];

//uint16_t tea5764_pll_value;
unsigned char tea5764_intmask;	//interrupt Mask for TEA5764
unsigned char RDS_Level;		//used to determin the quality of the Station


unsigned char  rds_data_ct[ RDS_CT_LEN ];               // CT Clock Time and Date
unsigned char rds_data[8];	 //data buffer for the recived rds data

bit isCT=0;					 //on if the current block is a CT
bit CTava=0;				 //Flag to main if new CT data is avalble
bit isError=0;				 //bad data recived



//save the current frequency for restart
unsigned char TEASaveFrequenzy(void)
{	  
unsigned tmp_fr=tea5764_GetFreq();  //get current frequency
code unsigned int* pFr_Save=&Fr_Save;  //address in FlASH where to save

if(tmp_fr!=Fr_Save) //if chaned save new  pll frequency seting
	return flash_program ((unsigned int)pFr_Save,&tmp_fr,2);
return 0;
}


void ProcessTEAInterrupt(void)
{
//RDS Data avalible
if((tea5764_fm_buf[REG_INTFLAG]	&0x80)==0x80) //DAVFLG
	{
	 if(0==TEA_ProcessRDSData())
	 	if(RDS_Level<255)
  		   RDS_Level++;
	}
}




// Read FM Registers and store it
void tea5764_read_fm_registers( void )
{
    i2c_read_bytes( TEA5764_FM, tea5764_fm_buf, TEA5764_FM_RCOUNT );
}

// Write FM Registers to Device
void tea5764_write_fm_registers( void )
{
	unsigned char buffer[TEA5764_FM_WCOUNT ];

    unsigned char i = 0;
    buffer[ i++ ] = tea5764_intmask;
    buffer[ i++ ] = tea5764_fm_buf[ REG_FREQSETMSB ];
    buffer[ i++ ] = tea5764_fm_buf[ REG_FREQSETLSB ];
    buffer[ i++ ] = tea5764_fm_buf[ REG_TNCTRL1 ];	  //currently not used
    buffer[ i++ ] = tea5764_fm_buf[ REG_TNCTRL2 ];
	/*	curently not used
    buffer[ i++ ] = tea5764_fm_buf[ REG_TESTBITS ];
    buffer[ i++ ] = tea5764_fm_buf[ REG_TESTMODE ];*/

    i2c_write_bytes( TEA5764_FM, buffer, TEA5764_FM_WCOUNT );
}


// read need RDS Registers 
void tea5764_read_rds_registers( void )
{
    i2c_read_bytes( TEA5764_RDS, tea5764_rds_buf, TEA5764_RDS_RCOUNT );
}

// write RDS Registers to Device
/* curently not used
void tea5764_write_rds_registers( void )
{
	unsigned char buffer[TEA5764_RDS_WCOUNT ];
    // Ummapen auf write registers siehe oben
    buffer[ 0 ] = tea5764_rds_buf[ REG_RDSCTR1 ];
    buffer[ 1 ] = tea5764_rds_buf[ REG_RDSCTR2 ];
    buffer[ 2 ] = tea5764_rds_buf[ REG_PAUSEDET ];
    buffer[ 3 ] = tea5764_rds_buf[ REG_RDSBBL ];
    i2c_write_bytes( TEA5764_RDS, buffer, TEA5764_RDS_WCOUNT );
} */ 

// read the interrupt register
void tea5764_read_int_registers( void )
{
    i2c_read_bytes( TEA5764_FM, tea5764_fm_buf, 2 );
}


//set PLL to given freq.
void tea5764_SetPllFreq(unsigned int frq)
{
	RDS_Level=0;
  	tea5764_fm_buf[ REG_FREQSETMSB ]=frq>>8;
  	tea5764_fm_buf[ REG_FREQSETLSB ]=frq&0xFF;
	tea5764_write_fm_registers();
	do 		//Wait for search done
		{
		Delay10ms(10);
		tea5764_read_int_registers();
		}while((tea5764_fm_buf[REG_INTFLAG]&0x2)!=0x2);	//state "mc ready"
}

//find next station
bit tea5764_search(unsigned int frq,bit dir )
{
	frq=((unsigned long)frq*FREQ_MUL+FREQ_OFF)/FREQ_REF; //convert to PLL value
	if(dir)
		frq|=0xC000;	//set search bit and direction bit
	else
		frq|=0x4000;

	//make shure high side injection is on
	tea5764_fm_buf[REG_TNCTRL2]|=0x10;	
	//Start a search
	tea5764_SetPllFreq(frq);
	 //end of band rang?
	if ((tea5764_fm_buf[REG_INTFLAG]&0x1)==0x1)  //BLFLAG
		{
		dir=!dir; //change direction										   U
		}
	tea5764_read_fm_registers();

return dir;

}

//set the reciver to the given freq.
void tea5764_SetFreq(unsigned int frq)
{
   	 //calculate PLL values	 and set PLL
	tea5764_SetPllFreq(((unsigned long)frq*FREQ_MUL+FREQ_OFF)/FREQ_REF);
}

//return the curent PLL seting
unsigned int tea5764_GetPllFreq(void)
{
  return (tea5764_fm_buf[ REG_FREQCHKMSB ]<<8)|(tea5764_fm_buf[ REG_FREQCHKLSB ]);
} 

//return the current ferq. that is set
unsigned int tea5764_GetFreq(void)
{
//round to next 100khz
return ((((unsigned long)tea5764_GetPllFreq()*FREQ_REF-FREQ_OFF)/(FREQ_MUL/2))+1)/2;
}


//initialize the radio
void tea5764_init( void )
{
    // Inital defaults
    tea5764_intmask = INTMSK;
	RDS_Level=0;
    tea5764_read_fm_registers();
	tea5764_fm_buf[ REG_TNCTRL1 ]=0x8; //power off
    tea5764_write_fm_registers();
	Delay10ms(2);	  //wait a moment

	i2c_write_bytes( TEA5764_FM, FM_InitData, TEA5764_FM_WCOUNT );
	Delay10ms(2);	 //wait TEA to go on
	i2c_write_bytes( TEA5764_RDS, RDS_InitData, TEA5764_RDS_WCOUNT );

    // read  FM Registers 
    tea5764_read_fm_registers();
    // read  RDS Registers
    tea5764_read_rds_registers();


}


bit TEA_ProcessRDSData(void)
{
	 tea5764_read_rds_registers();
	 if((tea5764_rds_buf[REG_RDSSTAT1]&0x3)==3) //check that no errors
			{ 
			isCT=0;
			isError=1;
			return isError;
			}
	//get Block (A,b,C,D) and switch based on Block type
	switch( (tea5764_rds_buf[ REG_RDSSTAT1 ] >> 4 ) )
		{
		case RDS_BLK_A:	   //store data
			rds_data[0]=tea5764_rds_buf[ REG_RDSLBMSB ];
			rds_data[1]=tea5764_rds_buf[ REG_RDSLBLSB ];
			isError=0;
			break;
		case RDS_BLK_B:
			//store data
			rds_data[2]=tea5764_rds_buf[ REG_RDSLBMSB ];
			rds_data[3]=tea5764_rds_buf[ REG_RDSLBLSB ];

            // 4A  Date Time Groupe Dedect (CT)	  Bit 11 to 15 = Group
			//is it a CT Group
	    	if(  (tea5764_rds_buf[ REG_RDSLBMSB ] >> 3) == RDS_GRP_A_CTDO )
           		{
			 	isCT=1;
			 	CTava=0;
                rds_data_ct[ 0 ] = tea5764_rds_buf[ REG_RDSLBLSB ]; //store CT data
            	}
			else
				isCT=0;
			break;
		case RDS_BLK_C:
			//store data
			rds_data[4]=tea5764_rds_buf[ REG_RDSLBMSB ];
			rds_data[5]=tea5764_rds_buf[ REG_RDSLBLSB ];
			//is it a CT Group
			if(isCT)
				{
                rds_data_ct[ 1 ] = tea5764_rds_buf[ REG_RDSLBMSB ];
                rds_data_ct[ 2 ] = tea5764_rds_buf[ REG_RDSLBLSB ];
				}
			break;
		case RDS_BLK_D:
			//store data
			rds_data[6]=tea5764_rds_buf[ REG_RDSLBMSB ];
			rds_data[7]=tea5764_rds_buf[ REG_RDSLBLSB ];
			//Grop compleet A to D shall we sen RAW data
			if(SentRawRDS && !isError)
				{		 //sent out to UART
				uart_transmit(0x3F);
				uart_transmit(rds_data[ 0 ]);
				uart_transmit(rds_data[ 1 ]);
				uart_transmit(rds_data[ 2 ]);
				uart_transmit(rds_data[ 3 ]);
				uart_transmit(rds_data[ 4 ]);
				uart_transmit(rds_data[ 5 ]);
				uart_transmit(rds_data[ 6 ]);
				uart_transmit(rds_data[ 7 ]);
				uart_transmit(0x3F);
			} 
			//is it a CT Group
			if(isCT)
				{
             	rds_data_ct[ 3 ] = tea5764_rds_buf[ REG_RDSLBMSB ];
                rds_data_ct[ 4 ] = tea5764_rds_buf[ REG_RDSLBLSB ];
				isCT=0;
				CTava=1;
				}
			break;
		default:
			break;	
		}//end of switch
return isError;
}
