#ifndef _TEA5764H_
#define _TEA5764H_


// FM Functions
void tea5764_read_rds_registers( void );  //read TEA RDS registers to buffer
void tea5764_write_rds_registers( void ); //write TEA RDS registers from buffer
void tea5764_read_int_registers( void );  //read TEA interrupt registers to buffer
void tea5764_read_fm_registers( void );	  //read TEA FM registers to buffer
void tea5764_init( void );				  //initialize
void tea5764_write_fm_registers( void );   //write TEA FM registers from buffer
bit tea5764_search(unsigned int,bit dir);	 //search next radio station
void tea5764_SetFreq(unsigned int frq);		  //set pll frequency
unsigned int tea5764_GetFreq(void);
void ProcessTEAInterrupt(void);
unsigned char TEASaveFrequenzy(void);


#define FREQ_OFF 56250L
#define FREQ_REF 2048L
#define FREQ_MUL 25000L



//--------------------------------------------------------
// TEA5764 DEFINES
//--------------------------------------------------------
#define TEA5764_FM              0x10
#define TEA5764_RDS             0x11

#define TEA5764_FM_RCOUNT       8 //12
#define TEA5764_FM_WCOUNT       5 //7                     
#define TEA5764_RDS_RCOUNT      8 //12
#define TEA5764_RDS_WCOUNT      4
#define TEA5764_FM_WCOUNT2      7                     


// Register defaults fuer FM Registers
enum TEA5764_FM_DEFAULTS
{
    INTMSK      = 0x00,
    FRQSETMSB   = 0x80,	 //search up search  start at 87.5 
    FRQSETLSB   = 0x00,
    TNCTRL1     = 0xC8, //FM&RDS on if count 15,02ms 
    TNCTRL2     = 0xD2, //High side injection ADC7  hard mute, de-emphasis 50ms
    TESTBITS    = 0x10, //RFAGC, LDX off ,large hys. 
    TESTMODE    = 0x00  //
};

// Register defaults fuer RDS Registers
enum TEA5764_RDS_DEFAULTS
{
    RDSCTRL1    = 0x20, // 0x20 coorection
    RDSCTRL2    = 0x10,
    PAUSEDET    = 0x00,
    RDSBBL      = 0x2A
};


										  // Register positionen READ Only Registers
enum TEA5764_REGISTERS_FM
{
    REG_INTFLAG,
    REG_INTMSK,
    REG_FREQSETMSB,
    REG_FREQSETLSB,
    REG_TNCTRL1,
    REG_TNCTRL2,
    REG_FREQCHKMSB,
    REG_FREQCHKLSB,
    REG_IFCHK,
    REG_LEVCHK,
    REG_TESTBITS,
    REG_TESTMODE
};

enum TEA5764_REGISTERS_RDS
{
    REG_RDSSTAT1,
    REG_RDSSTAT2,
    REG_RDSLBMSB,
    REG_RDSLBLSB,
    REG_RDSPBMSB,
    REG_RDSPBLSB,
    REG_RDSBBC,
    REG_RDSGBC,
    REG_RDSCTR1,
    REG_RDSCTR2,
    REG_PAUSEDET,
    REG_RDSBBL
    // rest noch nachziehen bei bedarf
};


// Buffer size

extern idata unsigned char tea5764_fm_buf[ TEA5764_FM_RCOUNT ];
extern idata unsigned char tea5764_rds_buf[ TEA5764_RDS_RCOUNT ];
extern code unsigned int Fr_Save;
extern unsigned char tea5764_state;

//uint16_t tea5764_pll_value;
extern unsigned char tea5764_intmask;
extern unsigned char RDS_Level;


#endif //_TEA5764H_

#ifndef _RDSH_
#define _RDSH_

#define RDS_CT_LEN 5

bit TEA_ProcessRDSData(void);

extern unsigned char  rds_data_ct[ RDS_CT_LEN ]; 
extern bit CTava;	//set by DRS decoder if CT data has been recived
extern bit SentRawRDS;

enum RDS_BLOCKS
{
    RDS_BLK_A,          // Block A
    RDS_BLK_B,          // Block B
    RDS_BLK_C,          // Block C
    RDS_BLK_D,          // Block D
    RDS_BLK_C_,         // Block C'
    RDS_BLK_E,          // Block E
    RDS_BLK_INVALID_E,  // Block Invalid E 
    RDS_BLK_INVALID     // Block Invalid
};

/**
 * \brief Rds Groups
 */
enum RDS_GROUPS
{
    RDS_GRP_A_BTSI=0,     //  0A, Basic tuning and switching information only
    RDS_GRP_B_BTSI,     //  0B, Basic tuning and switching information only
    RDS_GRP_A_PINSLCO,  //  1A, Program Item Number and slow labeling code only
    RDS_GRP_B_PIN,      //  1B, Program ItemNumber
    RDS_GRP_A_RTO,      //  2A, Radio Text Only
    RDS_GRP_B_RTO,      //  2B, Radio text only
    RDS_GRP_A_AIODA,    //  3A, Applications Identification for ODA only
    RDS_GRP_B_ODA3,     //  3B, Open Data Applications
    RDS_GRP_A_CTDO,     //  4A, Clock-time date only
    RDS_GRP_B_ODA4,     //  4B, Open Data Applications
    RDS_GRP_A_TDCODA,   //  5A, Transparent Data Chanels (32 channels) or ODA
    RDS_GRP_B_TDCODA,   //  5B, Transparent Data Chanels (32 channels) or ODA
    RDS_GRP_A_IHOODA,   //  6A, In House applications or ODA
    RDS_GRP_B_IHOODA,   //  6B, In House applications or ODA
    RDS_GRP_A_RPODA,    //  7A, Radio Paging or ODA
    RDS_GRP_B_ODA7,     //  7B, Open Data Applications
    RDS_GRP_A_TMCODA,   //  8A, Traffic Message Channel or ODA
    RDS_GRP_B_ODA,      //  8B, Open Data Applications
    RDS_GRP_A_EWSODA,   //  9A, Emergency Warning System or ODA
    RDS_GRP_B_ODA9,     //  9B, Open Data Applications
    RDS_GRP_A_PTYN,     // 10A, Program Type Name
    RDS_GRP_B_ODA10,    // 10B, Open Data Applications
    RDS_GRP_A_ODA,      // 11A, Open Data Applications
    RDS_GRP_B_ODA11,    // 11B, Open Data Applications
    RDS_GRP_A_ODA12,    // 12A, Open Data Applications
    RDS_GRP_B_ODA12,    // 12B, Open Data Applications
    RDS_GRP_A_ERPODA,   // 13A, Enhanced Radio Paging or ODA
    RDS_GRP_B_ODA13,    // 13B, Open Data Applications
    RDS_GRP_A_EON,      // 14A, Enhanced Other Networks information only
    RDS_GRP_B_EON,      // 14B, Enhanced Other Networks information only
    RDS_GRP_A_DRBDS,    // 15A, Defined in RBDS only
    RDS_GRP_B_FSI       // 15B, Fast switching information only
};


#endif //_RDSH_