/***********************************************************************
MODULE:    Timers
VERSION:   1.00
CONTAINS:  Routines for controlling the timers on the Philips
           P89LPC921
COPYRIGHT: Embedded Systems Academy, Inc. - www.esacademy.com
LICENSE:   May be freely used in commercial and non-commercial code
           without royalties provided this copyright notice remains
           in this file and unaltered
WARNING:   IF THIS FILE IS REGENERATED BY CODE ARCHITECT ANY CHANGES
           MADE WILL BE LOST. WHERE POSSIBLE USE ONLY CODE ARCHITECT
           TO CHANGE THE CONTENTS OF THIS FILE
GENERATED: On "Apr 03 2010" at "00:53:19" by Code Architect 2.13
***********************************************************************/

// SFR description needs to be included
#include <reg921.h>
#include <timers.h>
#include <flash.h>


code unsigned int RelSave=0xDC00; 	//saved value for the reload value

unsigned char delayX;	 		//used by delay_10ms
bit delayOn;

unsigned char DcfPulse=0;		//used to gen DCF 100/200 ms pulses
bit DcfPulseOn;

bit S_Tic;					   //once per second
bit S_Tic2;
bit M_Tic;					   //once per minute
bit M_Tic2;

unsigned char ticcnt;		  //couter to determin 1 second
unsigned char second;		  //count the seconds
bit reset;					  //set it timer shall be reset 
signed int OneMT;			  //used for RC osc calibration	
unsigned int Reload;		  //reload value


//save the current reload value to flash
unsigned char TimerSaveReload(void)
{
code unsigned int* pRelSave=&RelSave;//get address

	  //if chaned save new  pll frequency seting
if(Reload!=RelSave)
	return flash_program ((unsigned int)pRelSave,&Reload,2);
return 0;
}



void Init_Timer(void)
{
/*DESC:    Initializes timers
         Timer 0 generates an interrupt every 10ms
         Timer 1 is not used*/


//init data
char delayX=0;
delayOn=0;

DcfPulse=0;
DcfPulseOn=0;

S_Tic=0;
S_Tic2=0;
M_Tic=0;
M_Tic2=0;

ticcnt=0;
second=0;
reset=0;
OneMT=0;
Reload=RelSave;


  // configure timer 0
  TMOD &= 0xF0;
  TMOD |= 0x01;
  TAMOD &= 0xFE;

  // timer values
  TH0 = Reload>>8;
  TL0 = Reload&0xFF;

  // set timer 0 isr priority to 0
  IP0 &= 0xFD;
  IP0H &= 0xFD;

  // enable timer 0 interrupt
  ET0 = 1;

  // start timer 0
  TR0 = 1;
}
/***********************************************************************
DESC:    Timer 0 Interrupt Service Routine
RETURNS: Nothing																   	
CAUTION: timers_init must be called first
         EA must be set to 1
called every 10ms -->1/100s
************************************************************************/

void timers_isr0 (void) interrupt 1 using 1
{
  // reinitialize timer register
  TH0 = Reload>>8;
  TL0 = Reload&0xFF;
  
  OneMT--;	  //count down the one Minute couter used for calibration

  if(delayOn)	//used by Delay10ms()
  	{
	if(delayX==0)
		delayOn=0;
	else
		delayX--;
	}

  if(DcfPulseOn)  //gennerate DCF Pulse
  	{
  	TxD=0;
  	if(!DcfPulse--)
		{
  		DcfPulseOn=0;
	  	TxD=1;
		}
	}
		
  if(ticcnt==20)	
  	{
 	S_Tic2=0;
	}

  ticcnt++;	

  if((ticcnt >= 100) || reset)	//one second (100 x 10ms) is gone
  	{
	S_Tic=!S_Tic2;
	S_Tic2=1;
	ticcnt=0;
	if(second==1)
		M_Tic2=0;		
	second++;
	if((second>=60)|| reset)   //one minute is gone
		{
		M_Tic=!M_Tic2;
		M_Tic2=1;
		second=0;
		}
	reset=0;
	}
} // timers_isr0
	

signed int ResetS_Count(void)	 //reset timer (used for sync)
{
signed int ret=OneMT;
OneMT=6000;
reset=1;
return ret;
}