
/***********************************************************************
MODULE:    UART
VERSION:   1.04
CONTAINS:  Routines for controlling the UART peripheral on the Philips
           P89LPC921
COPYRIGHT: Embedded Systems Academy, Inc. - www.esacademy.com
LICENSE:   May be freely used in commercial and non-commercial code
           without royalties provided this copyright notice remains
           in this file and unaltered
WARNING:   IF THIS FILE IS REGENERATED BY CODE ARCHITECT ANY CHANGES
           MADE WILL BE LOST. WHERE POSSIBLE USE ONLY CODE ARCHITECT
           TO CHANGE THE CONTENTS OF THIS FILE
GENERATED: On "Apr 03 2010" at "00:53:49" by Code Architect 2.13
***********************************************************************/

// SFR description needs to be included
#include <reg921.h>
#include "uart.h"
#include <util.h>
#include <Timeconvert.h>
#include <timers.h>

// flag that indicates if the UART is busy transmitting or not
static bit mtxbusy;
static bit mrxdata;

/***********************************************************************
DESC:    Initializes UART for mode 1
         Baudrate: 9600
RETURNS: Nothing
CAUTION: If interrupts are being used then EA must be set to 1
         after calling this function
************************************************************************/
void uart_init  ( void  )
{
  // configure UART
  // clear SMOD0
  PCON &= ~0x40;
  SCON = 0x50;
  // set or clear SMOD1
  PCON &= 0x7f;
  PCON |= (0 << 8);
  SSTAT = 0x00;


  // configure baud rate generator
  BRGCON = 0x00;
  BRGR0 = 0xB0;
  BRGR1 = 0x00;
  BRGCON = 0x03;
  // enable break detect
  AUXR1 |= 0x40;


  // TxD = push-pull, RxD = input
  P1M1 &= ~0x01;
  P1M2 |= 0x01;
  P1M1 |= 0x02;
  P1M2 &= ~0x02;

  // initially not busy
  mtxbusy = 0;

  //nothing recived jet
  mrxdata=0;
	
  // set isr priority to 0
  IP0 &= 0xEF;
  IP0H &= 0xEF;
  // enable uart interrupt
  ES = 1;

} // uart_init

/***********************************************************************
DESC:    UART Interrupt Service Routine
RETURNS: Nothing
CAUTION: uart_init must be called first
         EA must be set to 1
************************************************************************/
void uart_isr   (  void  ) interrupt 4 using 1
{
  if (RI)
  {
    // clear interrupt flag
    RI = 0;
	mrxdata=1;
  } // if

  if (TI)
  {
    // clear interrupt flag
    TI = 0;
    // no longer busy
    mtxbusy = 0;
  } // if

} // uart_isr

/***********************************************************************
DESC:    Transmits a 8-bit value via the UART in the current mode
         May result in a transmit interrupt if enabled.
RETURNS: Nothing
CAUTION: uart_init must be called first
************************************************************************/
void uart_transmit
  (
  unsigned char value    // data to transmit
  )
{
  while(mtxbusy);
  mtxbusy = 1;
  SBUF = value;
} // uart_transmit


/***********************************************************************
DESC:    Gets a receive status from UART
RETURNS: status
CAUTION: uart_init must be called first
***********************************************************************/
bit uart_peek(void)
{
return mrxdata;
}


/***********************************************************************
DESC:    Gets a received 8-bit value from the UART
RETURNS: Received data
CAUTION: uart_init must be called first
************************************************************************/
unsigned char uart_get  (  void  )
{
  mrxdata=0;
  return SBUF;
} // uart_get



void SendAsHex(unsigned char val)
{
	uart_transmit(htoa_hi(val));
	uart_transmit(htoa_lo(val));
} 


#ifdef DEBUG


void SendCRLF(void)
{
	uart_transmit('\r');
	uart_transmit('\n');
} 

void SendAsHexWSpace(unsigned char val)
{
SendAsHex(val);
uart_transmit(' ');
} 



 #endif